import CodeMirror from 'codemirror';
import 'codemirror/addon/hint/show-hint';
import { GraphQLNonNull, GraphQLList, } from 'graphql';
import { getAutocompleteSuggestions, Position } from 'graphql-language-service';
import escapeHTML from 'escape-html';
CodeMirror.registerHelper('hint', 'graphql', (editor, options) => {
    const schema = options.schema;
    if (!schema) {
        return;
    }
    const cur = editor.getCursor();
    const token = editor.getTokenAt(cur);
    const tokenStart = token.type !== null && /"|\w/.test(token.string[0])
        ? token.start
        : token.end;
    const position = new Position(cur.line, tokenStart);
    const rawResults = getAutocompleteSuggestions(schema, editor.getValue(), position, token, options.externalFragments);
    const results = {
        list: rawResults.map(item => ({
            text: item.label,
            description: item.documentation,
            isDeprecated: item.isDeprecated,
            deprecationReason: item.deprecationReason,
            render: (el) => {
                el.innerHTML = `<span class="info-label">${item.label}</span> <span class="CodeMirror-hint-information"><span class="infoType">${item.type && renderType(item.type)}</span></span>`;
            },
        })),
        from: { line: cur.line, ch: tokenStart },
        to: { line: cur.line, ch: token.end },
    };
    if (results && results.list && results.list.length > 0) {
        results.from = CodeMirror.Pos(results.from.line, results.from.ch);
        results.to = CodeMirror.Pos(results.to.line, results.to.ch);
        CodeMirror.signal(editor, 'hasCompletion', editor, results, token);
    }
    return results;
});
function renderType(type) {
    if (type instanceof GraphQLNonNull) {
        return `${renderType(type.ofType)}!`;
    }
    if (type instanceof GraphQLList) {
        return `[${renderType(type.ofType)}]`;
    }
    return `<a class="typeName">${escapeHTML(type.name)}</a>`;
}
//# sourceMappingURL=hint.js.map